<?xml version='1.0'?>
<!-- vim: sw=2 sta et
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:doc="http://nwalsh.com/xsl/documentation/1.0"
                xmlns:l="http://docbook2x.sourceforge.net/xsl/localization"
                exclude-result-prefixes="doc l"
                version='1.0'
                xml:lang="en">

<!-- ********************************************************************
     $Id: xref.xsl,v 1.12 2004/08/12 19:53:23 stevecheng Exp $
     ********************************************************************

     (C) 2000-2004 Steve Cheng <stevecheng@users.sourceforge.net>

     This file is part of the docbook2X XSLT stylesheets for
     converting DocBook to man pages.

     See ../../COPYING for the copyright status of this software.

     ******************************************************************** -->

<doc:reference xmlns="">
<title>Handle cross references and links</title>
</doc:reference>

<!-- ==================================================================== -->

<xsl:template match="anchor">
  <!-- Don't do anything, man pages have no hyperlinks -->
</xsl:template>

<!-- ==================================================================== -->

<xsl:key name="id"
         match="*[@id]"
         use="@id" />

<xsl:template match="xref">
  <xsl:variable name="target" select="key('id', @linkend)"/>

  <xsl:call-template name="check-idref">
    <xsl:with-param name="target" select="$target" />
    <xsl:with-param name="bad-content">
      <xsl:call-template name="gentext-rendering">
        <xsl:with-param name="key" select="'dangling-xref'" />
        <xsl:with-param name="content" select="string(@linkend)" />
      </xsl:call-template>
    </xsl:with-param>
    <xsl:with-param name="content">
      <xsl:apply-templates select="$target" mode="xref-to" />
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

        
<!-- ==================================================================== -->

<xsl:template match="link">
  <xsl:variable name="target" select="key('id', @linkend)"/>

  <xsl:call-template name="check-idref">
    <xsl:with-param name="target" select="$target" />
    <xsl:with-param name="bad-content">
      <xsl:call-template name="gentext-rendering">
        <xsl:with-param name="key" select="'dangling-xref'" />
        <xsl:with-param name="content" select="string(@linkend)" />
      </xsl:call-template>
    </xsl:with-param>
    <xsl:with-param name="content">
      <xsl:call-template name="gentext-rendering">
        <xsl:with-param name="key" select="'man-link'" />
        <xsl:with-param name="content">
          <xsl:apply-templates select="$target" mode="xref-to" />
        </xsl:with-param>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template match="ulink">

  <xsl:choose>
    <xsl:when test="(starts-with(., 'http://') or
                     starts-with(., 'ftp://')) and 
                     string(.) = @url">
      <!-- Often seen from man pages converted by doclifter -->
      <xsl:call-template name="gentext-rendering">
        <xsl:with-param name="key" select="'man-url'" />
        <xsl:with-param name="content" select="string(@url)" />
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:apply-templates />
      <xsl:call-template name="gentext-rendering">
        <xsl:with-param name="key" select="'man-ulink'" />
        <xsl:with-param name="content" select="string(@url)" />
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- ==================================================================== -->

<doc:mode mode="xref-to" xmlns="">
<refpurpose>Give cross-reference markup</refpurpose>
<refdescription>
<para>
Processing an element with this mode returns the markup/text that should
be used for referring to it in cross-references.</para>
</refdescription>
</doc:mode>

<xsl:template match="*" mode="xref-to">
  
  <xsl:variable name="refentry" select="ancestor-or-self::refentry[1]" />

  <xsl:choose>
    <xsl:when test="count($refentry) != 0">
      <!-- Get name of man page -->
      <b>
        <xsl:apply-templates select="$refentry" mode="for-title" />
      </b>
      <xsl:text>(</xsl:text>
      <xsl:value-of select="$refentry/refmeta/manvolnum" />
      <xsl:text>)</xsl:text>
    </xsl:when>

    <xsl:otherwise>
      <xsl:call-template name="gentext-rendering">
        <xsl:with-param name="key" select="'xref-non-refentry'" />
        <xsl:with-param name="content">
          <xsl:choose>
            <xsl:when test="@xreflabel">
              <xsl:value-of select="@xreflabel" />
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="." mode="for-title" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


</xsl:stylesheet>
